package com.example.entity;

import com.example.entity.dto.AccountDTO;
import lombok.Getter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.Collections;

@Getter
public class AccountDetails implements UserDetails {
    private final AccountDTO account;

    private AccountDetails(AccountDTO account) {
        this.account = account;
    }

    public static AccountDetailsBuilder withAccount(AccountDTO account) {
        return new AccountDetailsBuilder(account);
    }

    public static class AccountDetailsBuilder {
        private final AccountDTO account;

        private AccountDetailsBuilder(AccountDTO account) {
            this.account = account;
        }

        public AccountDetails build() {
            return new AccountDetails(account);
        }
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return Collections.singletonList(
                new SimpleGrantedAuthority("ROLE_" + account.role())
        );
    }

    @Override
    public String getPassword() {
        return account.password();
    }

    @Override
    public String getUsername() {
        return account.username();
    }
}
