package com.example.service.impl;

import com.example.entity.AccountDetails;
import com.example.entity.dto.AccountDTO;
import com.example.repository.AccountRepository;
import com.example.service.AccountService;
import org.springframework.security.core.userdetails.ReactiveUserDetailsService;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

@Service
public class AccountServiceImpl implements AccountService, ReactiveUserDetailsService {

    private final AccountRepository accountRepository;

    public AccountServiceImpl(AccountRepository accountRepository) {
        this.accountRepository = accountRepository;
    }

    /**
     * 实现 ReactiveUserDetailsService 接口的方法
     */
    @Override
    public Mono<UserDetails> findByUsername(String username) {
        return findAccountByNameOrEmail(username)
                .switchIfEmpty(Mono.error(new UsernameNotFoundException("此账号未注册")))
                .map(accountDTO -> AccountDetails
                        .withAccount(accountDTO)
                        .build());
    }

    /**
     * 实现 AccountService 接口的方法
     */
    @Override
    public Mono<AccountDTO> findAccountByNameOrEmail(String text) {
        return accountRepository.findByUsernameOrEmail(text, text);
    }
}
